# Copyright 2009 Antoine d'Otreppe de Bouvette
#
# This file is part of the Aspyct library. see http://www.aspyct.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''Aspyct '''

import sys

# Select the adequate version, and avoid syntax error with __import__
if sys.version_info[0] == 2:
    __import__('core', globals(), locals(), ['py2x'])
else:
    __import__('core', globals(), locals(), ['py3x'], 1)

# Make all core functions available
VERSION = '3.0 beta 4'
VERSION_INFO = (3, 0, 4)

def require(req):
    """ - where 'req' is a version string (x.x.x...)
    Raises an AssertionError if required version is higher than
    the current version.
    """
    req = req.split('.')
    cur = VERSION_INFO
    maxI = min(len(req), len(cur))
    for i in range(0, maxI):
        r = int(req[i])
        c = int(cur[i])
        if r > c:
            raise AssertionError('Required Aspyct version is not matched')
        elif c > r:
            return
    if len(req) > len(cur):
        raise AssertionError('Required Aspyct version is not matched')

__all__ = ["aop", "utils", "unit", "meta", "require"]
