# Copyright 2009 Antoine d'Otreppe de Bouvette
#
# This file is part of the Aspyct library. see http://www.aspyct.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import re

if sys.version_info[0] == 2:
    _imps = __import__('core.common', globals(), locals(), ['Aspect', 'wrap'])
else:
    _imps = __import__('core.common', globals(), locals(), ['Aspect', 'wrap'], 1)

Aspect = _imps.Aspect
wrap = _imps.wrap
del _imps

AspectType = type(Aspect)

class FakeAspect:
    def __call__(self, func):
        return func

class Pointcut(list):
    def __init__(self, ns, filter, match=True):
        list.__init__(self)
        
        for item in _search(ns, re.compile(filter), match):
            wrapped = wrap(getattr(ns, item))
            setattr(ns, item, wrapped)
            self.append(wrapped)
    
    def addAspect(self, aspect, *args, **kwargs):
        if type(aspect) is AspectType:
            for item in self:
                aspect(*args, **kwargs)(item)
        else:
            for item in self:
                aspect(item)

MethodType = type(Pointcut.addAspect)

def _search(ns, regexp, match):
    all = []
    for attrName in dir(ns):
        attr = getattr(ns, attrName)
        
        # If this item is not a function, pass
        if not type(attr) in (FunctionType, MethodType):
            continue
        
        if regexp.match(attrName):
            if match:
                all.append(attrName)
        elif not match:
            all.append(attrName)
    
    return all

def filter(pattern):
    return re.escape(pattern).replace(r'\*', r'\w*?')

FunctionType = type(filter)


__all__ = ("Aspect", "FakeAspect", "Pointcut", "filter")
