# Copyright 2009 Antoine d'Otreppe de Bouvette
#
# This file is part of the Aspyct library. see http://www.aspyct.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class Singleton(type):
    '''Forces the class the be a singleton.
    An instance will be created the first time user tries to create an object,
    then this instance will always be returned instead of creating a new object.
    
    Usage example:
        class Single:
            __metaclass__ = Aspyct.meta.Singleton
            
            ...
        
        a = Single()
        b = Single()
        
        print(a is b)
    
    You may also provide arguments for the first instanciation.
        Single("Hi", "Hello")
    Subsequent "Single()" calls will accept arguments, but no new instance
    will be made
    '''
    def __init__(cls, *args, **kwargs):
        type.__init__(cls, *args, **kwargs)
        cls.__singleton_inst = None
    
    def __call__(cls, *args, **kwargs):
        if not cls.__singleton_inst:
            cls.__singleton_inst = cls.__new__(cls)
            cls.__init__(cls.__singleton_inst, *args, **kwargs)
        
        return cls.__singleton_inst
